@@ -1,5 +1,5 @@
-# GNU Solfege - eartraining for GNOME
-# Copyright (C) 2000, 2001, 2002, 2003  Tom Cato Amundsen
+# GNU Solfege - ear training for GNOME
+# Copyright (C) 2000, 2001, 2002, 2003, 2004  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,10 +29,10 @@ class App(gtk.Window):
         self._cid = 4
         self.__vbox = gtk.VBox()
         self.add(self.__vbox)
-        self.__menubox = gu.bHBox(self.__vbox, gtk.FALSE)
-        self.__toolbarbox = gu.bHBox(self.__vbox, gtk.FALSE)
+        self.__menubox = gu.bHBox(self.__vbox, False)
+        self.__toolbarbox = gu.bHBox(self.__vbox, False)
         self.__contentsbox = gu.bVBox(self.__vbox)
-        self.__appbarbox = gu.bHBox(self.__vbox, gtk.FALSE)
+        self.__appbarbox = gu.bHBox(self.__vbox, False)
         self.__dockitems = {}
     def show(self):
         self.__vbox.show()
@@ -52,7 +52,7 @@ class App(gtk.Window):
         hdlbox.show()
         self.__dockitems['Menubar'] = hdlbox
 
-        self.__menubox.pack_start(hdlbox, gtk.TRUE, gtk.TRUE)
+        self.__menubox.pack_start(hdlbox, True, True)
         hdlbox.add(menu)
         menu.show()
     def add_toolbar(self, toolbar, name, behavior, placement, band_num,
@@ -67,52 +67,6 @@ class App(gtk.Window):
     def get_dock_item_by_name(self, name):
         return self.__dockitems[name]
 
-class PropertyBox(gtk.Dialog):
-    def __init__(self):
-        gtk.Dialog.__init__(self)
-        self.set_resizable(gtk.FALSE)
-        self.vbox.set_border_width(uiconsts.PAD)
-        self.vbox.set_spacing(uiconsts.PAD)
-        self.g_notebook = gtk.Notebook()
-        self.g_notebook.show()
-        self.vbox.pack_start(self.g_notebook, gtk.FALSE)
-
-        self.g_help = gtk.Button(stock='gtk-help')
-        self.g_help.connect('clicked', self.on_help)
-        self.action_area.pack_start(self.g_help)
-
-        self.g_apply = gtk.Button(stock='gtk-apply')
-        self.g_apply.connect('clicked', self._apply)
-        self.g_apply.set_sensitive(gtk.FALSE)
-        self.action_area.pack_start(self.g_apply)
-
-        self.g_close = gtk.Button(stock='gtk-close')
-        self.g_close.connect('clicked', lambda widget: self.destroy())
-        self.action_area.pack_start(self.g_close)
-
-        self.g_ok = gtk.Button(stock='gtk-ok')
-        self.g_ok.connect('clicked', self._ok)
-        self.g_ok.set_sensitive(gtk.FALSE)
-        self.action_area.pack_start(self.g_ok)
-    def connect(self, event, cb):
-        if event in ('apply', 'help'):
-                pass
-        else:
-                gtk.Dialog.connect(self, event, cb)
-    def append_page(self, page, label):
-        self.g_notebook.append_page(page, label)
-    def set_modified(self, m):
-        self.g_ok.set_sensitive(gtk.TRUE)
-        self.g_apply.set_sensitive(gtk.TRUE)
-    def _ok(self, widget):
-        self._apply(widget)
-        self.destroy()
-    def _apply(self, widget):
-        for x in range(len(self.g_notebook.get_children())):
-            self.on_apply(self.g_notebook.get_children()[x], x)
-        self.on_apply(self.g_notebook, -1)
-        self.g_apply.set_sensitive(gtk.FALSE)
-
 class GnomeCanvas(gtk.Label):
     def __init__(self):
         gtk.Label.__init__(self, "Not on windows")
@@ -130,27 +84,29 @@ class _ContribDialog(gtk.Dialog):
         for s in authors:
             l = gtk.Label(s)
             l.set_alignment(0, 0)
-            vbox.pack_start(l, gtk.FALSE)
-        nb.append_page(vbox, gtk.Label('Authors'))
+            vbox.pack_start(l, False)
+        nb.append_page(vbox, gtk.Label(_('Written by')))
         vbox = gtk.VBox()
         vbox.set_border_width(uiconsts.PAD)
         for s in documenters:
             l = gtk.Label(s)
             l.set_alignment(0, 0)
-            vbox.pack_start(l, gtk.FALSE)
-        nb.append_page(vbox, gtk.Label('Documenters'))
+            vbox.pack_start(l, False)
+        nb.append_page(vbox, gtk.Label(_('Documented by')))
         vbox = gtk.VBox()
         vbox.set_border_width(uiconsts.PAD)
         l = gtk.Label(translators)
         l.set_alignment(0, 0)
-        vbox.pack_start(l, gtk.FALSE)
-        nb.append_page(vbox, gtk.Label('Translators'))
+        vbox.pack_start(l, False)
+        nb.append_page(vbox, gtk.Label(_('Translated by')))
         self.get_children()[0].get_children()[-1].get_children()[0].grab_focus()
 
 class About(gtk.Dialog):
     def __init__(self, name, version, copyright, text, authors,
              documenters, translators, pixmap):
-        gtk.Dialog.__init__(self, 'About %s' % name, None, 0, ('Contributers', 2, 'gtk-close', 1))
+        gtk.Dialog.__init__(self, 'About %s' % name, None, 0)
+        self.add_button(_('C_redits'), 2)
+        self.add_button(gtk.STOCK_CLOSE, 1)
         self.m_name = name
         self.m_authors = authors
         self.m_documenters = documenters
@@ -168,7 +124,8 @@ class About(gtk.Dialog):
 
         self.vbox.pack_start(gtk.Label(text))
 
-        label = gtk.Label('<span size="smaller">%s</span>' % copyright)
+        s = copyright.replace("<", "&lt;").replace(">", "&gt;")
+        label = gtk.Label('<span size="smaller">%s</span>' % s)
         label.set_use_markup(1)
         self.vbox.pack_start(label)
         self.get_children()[0].get_children()[-1].get_children()[0].grab_focus()
