@@ -29,7 +29,7 @@ if __name__ == '__main__':
     sys.path.append(".")
     sys.path.append("src")
     import i18n
-    i18n.setup_srcdir()
+    i18n.setup(".")
 
 LOWEST_NOTENAME="c,,,,"
 HIGHEST_NOTENAME="g''''''"
@@ -114,9 +114,9 @@ def _test():
     import doctest, musicalpitch, duration, interval
     for mod in (duration, musicalpitch, interval, ):
         doctest.testmod(mod)
-        # FIXME calling setup_srcdir should not be necessary here.
+        # FIXME calling setup should not be necessary here.
         # But doctest.testmod does something strange with _.
-        i18n.setup_srcdir()
+        i18n.setup(".")
 
 if __name__ == '__main__':
     _test()
