@@ -18,7 +18,7 @@
 import gtk, gnome, gobject
 import gu
 import inputwidgets
-import widgets
+from specialwidgets import RandomTransposeDialog
 import soundcard, mpd
 import dataparser
 
@@ -104,18 +104,6 @@ class LessonbasedTeacher(Teacher):
         if self.m_statistics:
             self.m_statistics.lessonfile_changed(
              self.get_string('lessoncollection'), self.get_string('lessonfile'))
-    def __make_sure_existing_lessonfile_is_selected(self):
-        if not self.get_string('lessoncollection') or \
-               self.get_string('lessoncollection') \
-                 not in self.m_app.lessonfile_manager.get_collections():
-            self.set_string('lessoncollection', 'solfege')
-        F = self.m_app.lessonfile_manager.get_filenames(
-                 self.get_string('lessoncollection'), self.m_exname)
-        if F:
-            if self.get_string('lessonfile') not in F:
-                self.set_string('lessonfile', F[0])
-        else:
-            self.set_string('lessonfile','')
     def parse_lessonfile(self):
         self.m_question = None
         self.q_status = const.QSTATUS_NO
@@ -304,6 +292,16 @@ class Gui(gtk.VBox, cfg.ConfigUtils):
         print "dummy abstract.Gui.on_start_practise", self.m_exname
     def on_end_practise(self):
         print "dummy abstract.Gui.on_end_practise", self.m_exname
+    def handle_config_box_visibility(self):
+        """
+        Show self.config_box if it has any visible children, otherwise
+        hide it.
+        """
+        if filter(lambda c: c.get_property('visible'),
+                  self.config_box.get_children()):
+            self.config_box.show()
+        else:
+            self.config_box.hide()
     def on_key_press_event(self, widget, event):
         if (self.g_notebook is None or self.g_notebook.get_current_page() == 0) \
            and event.type == gtk.gdk.KEY_PRESS:
@@ -417,14 +415,10 @@ class IntervalGui(Gui):
 
 
 class LessonbasedGui(Gui):
-    def __init__(self, teacher, window):
-        Gui.__init__(self, teacher, window)
-    def do_change_lessonfile(self, w=None):
-        self.m_t.set_lessonfile(self.get_string('lessoncollection'),
-            self.get_string('lessonfile'))
-        self.update_gui_after_lessonfile_change()
+    def __init__(self, teacher, window, no_notebook=0):
+        Gui.__init__(self, teacher, window, no_notebook)
     def add_random_transpose_gui(self):
-        hbox = gu.bHBox(self.config_box, False, False)
+        self.g_random_transpose_box = hbox = gu.bHBox(self.config_box, False, False)
         label = gtk.Label(_("Random transpose:"))
         label.show()
         hbox.pack_start(label, False)
@@ -438,17 +432,10 @@ class LessonbasedGui(Gui):
         button.connect('clicked', self.run_random_transpose_dialog)
         hbox.pack_start(button)
     def run_random_transpose_dialog(self, widget):
-        dlg = widgets.RandomTransposeDialog(self.m_t.m_P.header.random_transpose, self.g_win)
+        dlg = RandomTransposeDialog(self.m_t.m_P.header.random_transpose, self.g_win)
         response = dlg.run()
         if response == gtk.RESPONSE_OK:
             self.m_t.m_P.header.random_transpose = dlg.get_value()
             self.g_random_transpose.set_text(str(self.m_t.m_P.header.random_transpose))
         dlg.destroy()
-    def add_select_lessonfile_gui(self):
-        self.g_lessonfile_selector = widgets.SelectLessonfileWidget(
-            self.m_t, self.g_win, self.update_gui_after_lessonfile_change)
-        self.g_lessonfile_selector.show()
-        self.config_box.pack_start(self.g_lessonfile_selector)
-
-
 
