@@ -15,20 +15,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-try:
-    app_running == True
-except:
-    import sys
-    sys.path.append("src")
-    sys.path.append(".")
-    import i18n
-    i18n.setup_srcdir()
-
-
 import configureoutput
 import errno, sys
 import shutil
-import signal
 import soundcard, mpd
 import lessonfile
 import statistics
@@ -36,6 +25,7 @@ import abstract
 import os
 import cfg, utils
 import webbrowser
+from osutils import *
 
 class RunExternalFailed(Exception):
     def __init__(self, v):
@@ -43,61 +33,30 @@ class RunExternalFailed(Exception):
     def __str__(self):
         return "\nRunning external program failed.\nPlease check your setup.\nThe command that failed was: '%s'" % (" ".join(self.v[1:]))
 
-__child_pid = None
-def run_external_program(cmdstring, wdir, argument):
-    global __child_pid
-    if "%s" not in cmdstring:
-        cmdline = "%s %s" % (cmdstring, argument)
-    else:
-        cmdline = cmdstring % argument
-    wdir = os.path.normpath(wdir)
-    cur_dir = os.getcwd()
-    v = cmdline.split()
-    v = v[:1] + v
-    if not os.path.exists(v[0]):
-        raise RunExternalFailed(v)
-        return
-    try:
-        if __child_pid is not None:
-            os.kill(__child_pid, signal.SIGKILL)
-            os.wait()
-        __child_pid = None
-    except OSError:
-        print "kill failed, OSError"
-    except AttributeError:
-        __child_pid = None
-    pid = os.fork()
-    if pid == 0:
-        os.chdir(wdir)
-        try:
-            os.execlp(*v)
-        except OSError, x:
-            print >> sys.stderr, "OS Error:", x
-            os._exit(-1)
-            os.chdir(cur_dir)
-        os.chdir(cur_dir)
-    else:
-        __child_pid = pid
+def linux_play_wav_file(filename):
+    run_external_program(cfg.get_string('sound/wav_player'), "", filename)
 
 class SolfegeApp(cfg.ConfigUtils):
-    def __init__(self, options, ui):
+    def __init__(self, options, ui, lessonfile_manager):
         """
         options -- command line options parsed by optparse
         """
         cfg.ConfigUtils.__init__(self, 'solfege-app')
+        # test_mode is when we are running a test from the Tests menu
+        self.m_test_mode = False
         self.m_options = options
         self.m_ui = ui
         self.check_rcfile()
         self.m_teachers = {}
         self.m_running_exercise = None
-        self.lessonfile_manager = lessonfile.LessonFileManager()
+        self.lessonfile_manager = lessonfile_manager
         self.m_sound_init_exception = None
         self.setup_sound()
     def check_rcfile(self):
         """See default.config for rcfileversion values, meanings and
         a description of how to add config variables.
         """
-        rcfileversion = 6
+        rcfileversion = 9
         if cfg.get_int("app/rcfileversion") > rcfileversion:
             cfg.drop_user_config()
             return
@@ -122,6 +81,17 @@ class SolfegeApp(cfg.ConfigUtils):
             cfg.set_string("mainwin/history_back_ak", "<alt>Left")
             cfg.set_string("mainwin/history_forward_ak", "<alt>Right")
             cfg.set_string("mainwin/history_reload_ak", "<ctrl>r")
+        if cfg.get_int("app/rcfileversion") <= 6:
+            cfg.set_list("config/lessoncollections", ['solfege', 'user'])
+        if cfg.get_int("app/rcfileversion") <= 7:
+            cfg.set_int("rhythm/countin_perc", 80)
+        if cfg.get_int("app/rcfileversion") <= 8:
+            cfg.del_key("singinterval/highest_tone")
+            cfg.del_key("singinterval/lowest_tone")
+            cfg.del_key("melodicinterval/highest_tone")
+            cfg.del_key("melodicinterval/lowest_tone")
+            cfg.del_key("harmonicinterval/highest_tone")
+            cfg.del_key("harmonicinterval/lowest_tone")
         cfg.set_int("app/rcfileversion", rcfileversion)
     def setup_sound(self):
         if sys.platform == 'win32' and \
@@ -143,6 +113,7 @@ class SolfegeApp(cfg.ConfigUtils):
             soundcard.initialise_external_midiplayer(
                     cfg.get_string("sound/midi_player"),
                     verbose_init=self.m_options.verbose_sound_init)
+            soundcard.play_wav_file = linux_play_wav_file
         elif cfg.get_string("sound/type") == '':
             self.m_ui.display_error_message(
 """You should configure sound
@@ -159,6 +130,7 @@ the preferences window.
                              cfg.get_int("sound/synth_number"),
                              cfg.get_string("sound/card_info"),
                              verbose_init=self.m_options.verbose_sound_init)
+                soundcard.play_wav_file = linux_play_wav_file
             except (soundcard.SoundInitException, OSError, ImportError), e:
                 self.m_sound_init_exception = e
                 soundcard.initialise_using_fake_synth(True)
@@ -201,91 +173,122 @@ It is also possible that the OS sound setup is incorrect.
     def please_help_me(self):
         if self.m_ui.m_viewer != 'docviewer':
             self.handle_href('%s-help.html' % self.m_ui.m_viewer)
+    def _practise_lesson_id(self, lesson_id, urlobj=None):
+        """
+        return the module name.
+        """
+        module = self.lessonfile_manager.get(lesson_id, 'module')
+        if self.m_running_exercise:
+            self.m_ui.box_dict[self.m_running_exercise].on_end_practise()
+        if module not in self.m_teachers:
+            self.create_teacher(module)
+            self.m_ui.initialise_exercise(self.m_teachers[module])
+        self.m_teachers[module].set_lessonfile(
+            self.lessonfile_manager.get(lesson_id, 'lessoncollection'),
+            self.lessonfile_manager.get(lesson_id, 'lessonfile'))
+        self.m_ui.activate_exercise(module, urlobj)
+        self.m_running_exercise = module
+        self.m_teachers[module].g_view = self.m_ui.box_dict[module]
+        self.m_ui.show_help_on_current()
+        return module
+    def practise_lesson_id(self, lesson_id):
+        module = self._practise_lesson_id(lesson_id)
+        self.m_ui.box_dict[module].on_start_practise()
+        self.m_ui.m_history.add("solfege:/%s" % lesson_id)
+        return module
+    def test_lesson_id(self, lesson_id):
+        self.m_test_mode = True
+        module = self.practise_lesson_id(lesson_id)
+        self.m_ui.box_dict[module].on_start_practise()
+        self.m_ui.enter_test_mode()
     def handle_href(self, href, display_docfile_set_adj=1):
-        if self.m_ui.m_viewer:
-            if display_docfile_set_adj == 1:
-                # we get here when we click on urls, because we want to
-                # save the place in the current document when we load the
-                # new one. We don't get here if handle_href is called from
-                # history_back.
-                self.m_ui.m_history.set_adj_of_current(self.m_ui.box_dict['docviewer'].get_vadjustment().value)
-        protocol, action, fn, anchor, lessoncollection, lessonfile, config = \
-           utils.parse_url(href)
-        if protocol is None:
-            if fn.endswith(".midi"):
-                if os.path.exists(os.path.join(self.m_ui.box_dict['docviewer'].m_htmlwidget.m_document_wd,fn)):
-                    midifilename = fn
-                elif len(fn.split('-')) > 1:
+        if self.m_ui.m_viewer == 'docviewer':
+            self.m_ui.m_history.set_adj_of_current(
+                self.m_ui.box_dict['docviewer'].get_vadjustment().get_value())
+        urlobj = utils.Url(href)
+        if not urlobj.protocol:
+            if urlobj.filename.endswith(".midi"):
+                if os.path.exists(os.path.join(self.m_ui.box_dict['docviewer'].m_htmlwidget.m_document_wd, urlobj.filename)):
+                    midifilename = urlobj.filename
+                elif len(urlobj.filename.split('-')) > 1:
                     # Filename-nl.midi => Filename.midi
-                    v = fn.split('-')
+                    v = urlobj.filename.split('-')
                     w = v[-1].split('.')
                     midifilename = "-".join(v[:-1])+"."+w[1]
                 else:
-                    midifilename = fn
+                    midifilename = urlobj.filename
                 run_external_program(self.get_string("sound/midi_player"),
                         self.m_ui.box_dict['docviewer'].m_htmlwidget.m_document_wd, midifilename)
             else:
-                self.m_ui.display_docfile(fn, anchor)
-        elif protocol == 'solfege':
-            self.handle_solfege_href(href, action, fn, anchor, lessoncollection,
-                                     lessonfile, config)
-        elif protocol == 'http':
+                self.m_ui.hide_help_on_current()
+                self.m_ui.display_docfile(urlobj.filename, urlobj.anchor)
+        elif urlobj.protocol == 'solfege':
+            self.handle_solfege_href(urlobj)
+        elif urlobj.protocol == 'http':
             webbrowser.open_new(href)
-        elif protocol == 'mailto':
+        elif urlobj.protocol == 'mailto':
             if self.get_string("config/mua"):
                 try:
-                    cmdline = self.get_string("config/mua") % href
+                    cmdline = self.get_string("config/mua") % urlobj.href
                 except TypeError:
-                    cmdline = "%s %s" % (self.get_string("config/mua"), href)
+                    cmdline = "%s %s" % (self.get_string("config/mua"), urlobj.href)
                 os.system(cmdline)
             else:
                 self.m_ui.dialog_ok(_("You have not selected email program. You can do so in the preferences window."))
         else:
-            print "unknown link type", protocol
-    def handle_solfege_href(self, href, action, fn, anchor, lessoncollection,
-                            lessonfile, config):
-        if action == 'all-lessonfiles':
+            print "unknown link type", urlobj.protocol
+    def handle_solfege_href(self, urlobj):
+        self.m_ui.hide_help_on_current()
+        if urlobj.action == 'all-lessonfiles':
             self.m_ui.display_html(self.lessonfile_manager.m_htmldoc)
             self.m_ui.m_history.add('solfege:all-lessonfiles')
             return
-        if lessonfile:
-            filepath = os.path.join(self.get_string('lessoncollections/%s' %lessoncollection), lessonfile)
-            if not os.path.isfile(filepath):
+        if urlobj.lessonfile:
+            filepath = os.path.join(self.get_string('lessoncollections/%s' % urlobj.lessoncollection), urlobj.lessonfile)
+            if not os.path.isfile(os.path.expanduser(filepath)):
                 self.m_ui.display_error_message("Lessonfile not found:\n%s" % filepath)
                 return
-        if (self.m_ui.m_viewer == 'docviewer'
-                   and self.m_running_exercise is not None
-                   and self.m_running_exercise != fn) or \
-                  (self.m_ui.m_viewer != 'docviewer'):
-            self.m_ui.box_dict[self.m_running_exercise].on_end_practise()
-            self.m_running_exercise = None
-        _old_viewer = self.m_ui.m_viewer
-        if fn not in self.m_teachers:
-            new_ex = True
-            n = utils.exercise_name_to_module_name(fn)
-            exec("import %s" % n)
-            if issubclass(locals()[n].Teacher, abstract.LessonbasedTeacher):
-                self.m_teachers[fn] = locals()[n].Teacher(fn, self, lessoncollection, lessonfile, config)
-            else:
-                self.m_teachers[fn] = locals()[n].Teacher(fn, self, config)
-            # if we don't have a teacher, then the Gui has not been
-            # initialized either
-            self.m_ui.initialise_exercise(self.m_teachers[fn])
-            self.m_teachers[fn].g_view = self.m_ui.box_dict[fn]
+        # the Url class does not know anything about lesson ids, so
+        # it reports it as lessoncollection without lessonfile.
+        if urlobj.lessoncollection and not urlobj.lessonfile:
+            self.practise_lesson_id(urlobj.lessoncollection)
+            return
+        p = lessonfile.LessonfileCommon(None)#FIXME it would have been
+        # better if we didn't have to parse the file twice.
+        p.parse_file(self.lessonfile_manager.get_complete_filename(
+            urlobj.lessoncollection, urlobj.lessonfile))
+        self._practise_lesson_id(p.header.lesson_id, urlobj)
+        # This loop changes variables in the lesson file header if they
+        # have set values in the url.
+        for key, val in urlobj.config.iteritems():
+            try:
+                self.m_teachers[p.header.module].m_P.header[key] = eval(val)
+            except SyntaxError:
+                self.m_ui.dialog_ok(_("Bad url parameter in link: '%s=%s'.\nTrying to continue anyway.") % (key, val))
+        self.m_ui.box_dict[p.header.module].on_start_practise()
+    def create_teacher(self, modulename):
+        """
+        Create the teacher in 'modulename' and add it to self.m_teachers.
+        """
+        exec("import %s" % modulename)
+        self.m_teachers[modulename] = locals()[modulename].Teacher(modulename, self)
+        return
+        if issubclass(locals()[modulename].Teacher, abstract.LessonbasedTeacher):
+            self.m_teachers[urlobj.exercisemodule] = locals()[modulename].Teacher(urlobj.exercisemodule, self, urlobj.lessoncollection, urlobj.lessonfile, urlobj.config)
         else:
-            new_ex = False
-            self.m_teachers[fn].configure_exercise(lessoncollection, lessonfile, config)
-        self.m_ui.activate_exercise(href)
-        if not (_old_viewer == 'docviewer'
-                and self.m_running_exercise is not None
-                and self.m_running_exercise == fn):
+            self.m_teachers[urlobj.exercisemodule] = locals()[modulename].Teacher(urlobj.exercisemodule, self, urlobj.config)
+    def reset_exercise(self, w=None):
+        """
+        Call on_end_practise, and then on_start_practise in
+        the currently active exercise, if we have a exercise.
+        """
+        if self.m_ui.m_viewer != 'docviewer':
+            self.m_ui.box_dict[self.m_ui.m_viewer].on_end_practise()
             self.m_ui.box_dict[self.m_ui.m_viewer].on_start_practise()
-        self.m_running_exercise = fn
     def quit_program(self):
         cfg.sync()
-        for teacher in self.m_teachers.itervalues():
-            if teacher.m_statistics is not None:
-                teacher.m_statistics.save_data()
+        if self.m_ui.m_viewer != 'docviewer':
+            self.m_ui.box_dict[self.m_ui.m_viewer].on_end_practise()
         if soundcard.synth:
             soundcard.synth.close()
 
