@@ -17,19 +17,8 @@
 
 import gtk
 import gobject
-
-__exercise_data__ = {
-    'exercise_name': 'chord-voicing',
-    'stock-icon': 'solfege-chord-voicing',
-    'menupath': '/Menubar/FileMenu/ChordsMenu',
-    'name': _('_Chord voicing'),
-}
-import exercise_setup
-exercise_setup.register_exercise(__exercise_data__)
-
 import gnome
-
-import gu, widgets
+import gu
 
 import mpd
 import mpd.musicdisplayer
@@ -43,16 +32,10 @@ import soundcard
 class Teacher(abstract.LessonbasedTeacher):
     OK = 0
     ERR_PICKY = 1
-    exercise_data = __exercise_data__
     #UGH should we do this here
-    def __init__(self, exname, app, lesson_collection, lesson_file, config):
-        abstract.LessonbasedTeacher.__init__(self, exname, app,
-            lesson_collection, lesson_file, config)
+    def __init__(self, exname, app):
+        abstract.LessonbasedTeacher.__init__(self, exname, app)
         self.lessonfileclass = lessonfile.ChordLessonfile
-        try: # try: because the file can have changed since startup
-            self.parse_lessonfile()
-        except dataparser.DataparserException, e:
-            self.m_app.m_ui.display_exception_message(e, __file__)
     def new_question(self):
         """
         return 1 if the teacher has a question to ask
@@ -168,9 +151,7 @@ class Gui(abstract.LessonbasedGui):
         # config_box #
         ##############
         self.config_box.set_spacing(gnome.ui.PAD_SMALL)
-        self.add_select_lessonfile_gui()
         self.add_random_transpose_gui()
-        self.update_gui_after_lessonfile_change()
     def new_question(self, widget=None):
         def exception_cleanup():
             soundcard.synth.stop()
@@ -208,16 +189,13 @@ class Gui(abstract.LessonbasedGui):
             self.g_win.display_question_music_error_message(
                self.m_t.m_P._idx, self.get_string('lessonfile'), e)
     def update_gui_after_lessonfile_change(self, *v):
-        if self.m_t.m_P:
-            self.g_random_transpose.set_text(str(self.m_t.m_P.header.random_transpose))
-            self.g_new.set_sensitive(True)
-            self.fill_chordtype_box()
-        else:
-            self.g_new.set_sensitive(False)
+        self.set_lesson_heading(self.m_t.m_P.header.lesson_heading)
+        self.g_random_transpose.set_text(str(self.m_t.m_P.header.random_transpose))
+        self.g_new.set_sensitive(True)
+        self.fill_chordtype_box()
         self.g_repeat.set_sensitive(False)
         self.g_repeat_arpeggio.set_sensitive(False)
         self.g_give_up.set_sensitive(False)
-        self.g_win.set_title("Solfege - " + self.get_pretty_name())
         self.clear_stacking_frame()
     def clear_stacking_frame(self):
         for c in self.g_source.get_children() + self.g_answer.get_children():
@@ -346,6 +324,7 @@ class Gui(abstract.LessonbasedGui):
         for c in self.g_chordtype_box.get_children():
             c.set_sensitive(c.get_data('chordtype')==t)
     def on_start_practise(self):
+        self.update_gui_after_lessonfile_change()
         self.g_flashbar.require_size([
             _("You have to solve this question first."),
             _("Type is already solved, now specify voicing."),
