@@ -20,10 +20,10 @@ import os
 import gobject, gtk, gnome, gnome.ui
 import gu, mpd
 import configureoutput
-import widgets
-from instrumentselector import nInstrumentSelector
+import notenamespinbutton
+from instrumentselector import nInstrumentSelector, InstrumentConfigurator
 import cfg
-import app
+import osutils
 import soundcard
 
 def find_wav_player_programs():
@@ -88,7 +88,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         box, category_vbox = gu.hig_category_vbox(_("Chord instruments"))
         page_vbox.pack_start(box, False)
         self.g_instrument_configurator  \
-              = widgets.InstrumentConfigurator("config", 3,
+              = InstrumentConfigurator("config", 3,
                     _("Use different instruments for chords and harmonic intervals."))
         category_vbox.pack_start(self.g_instrument_configurator, False)
         ########
@@ -101,20 +101,20 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         page_vbox.pack_start(box, False)
         sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
 
-        self.g_highest_singable = widgets.NotenameSpinButton(
+        self.g_highest_singable = notenamespinbutton.NotenameSpinButton(
             self.get_string('user/highest_pitch'))
         box = gu.hig_label_widget(_("Highest note user can sing:"),
                                   self.g_highest_singable,
                                   sizegroup)
         category_vbox.pack_start(box)
 
-        self.g_lowest_singable = widgets.NotenameSpinButton(
+        self.g_lowest_singable = notenamespinbutton.NotenameSpinButton(
             self.get_string('user/lowest_pitch'))
         box = gu.hig_label_widget(_("Lowest note user can sing:"),
                                   self.g_lowest_singable,
                                   sizegroup)
         category_vbox.pack_start(box)
-        widgets.nNotenameRangeController(
+        notenamespinbutton.nNotenameRangeController(
                   self.g_lowest_singable, self.g_highest_singable,
                   mpd.LOWEST_NOTENAME, mpd.HIGHEST_NOTENAME,
                   'user', 'lowest_pitch', 'highest_pitch')
@@ -330,10 +330,10 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         gu.bButton(category_vbox, _("_Apply changes and play test sound"), self.on_apply_and_play_test_sound)
 
     def test_wav_player(self, w):
-        app.run_external_program(self.g_wav_player.child.get_text(),
+        osutils.run_external_program(self.g_wav_player.child.get_text(),
                  'lesson-files/share', 'fifth-small-220.00.wav')
     def test_midi_player(self, w):
-        app.run_external_program(self.g_midi_player.child.get_text(),
+        osutils.run_external_program(self.g_midi_player.child.get_text(),
                         'lesson-files/share', 'fanfare.midi')
     def set_gui_from_config(self):
         if self.get_string("sound/type") == "fake-synth":
@@ -354,7 +354,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
     def on_apply_and_play_test_sound(self, *w):
         if self.on_apply() != -1:
             self.set_gui_from_config()
-        self.play_midi_test_sound()
+            self.play_midi_test_sound()
     def play_midi_test_sound(self):
         mpd.play_music(r"""
         \staff\relative c{
