@@ -17,20 +17,12 @@
 
 from __future__ import division
 
-__exercise_data__ = {
-    'exercise_name': 'harmonic-interval',
-    'stock-icon': 'solfege-harmonic-interval',
-    'menupath': '/Menubar/FileMenu/IntervalsMenu',
-    'name': _('_Harmonic interval'),
-}
-import exercise_setup
-exercise_setup.register_exercise(__exercise_data__)
-
 import os
 import pickle
 import time
 import gtk, gnome, gobject
-import gu, widgets, inputwidgets
+import gu, inputwidgets
+from multipleintervalconfigwidget import IntervalCheckBox
 import soundcard, mpd
 import abstract, const, utils
 import statistics, statisticsviewer
@@ -38,19 +30,25 @@ import htmlwidget
 import cfg
 import random
 import gethomedir
-import intervaltest
+import lessonfile
 
-class Teacher(abstract.Teacher):
-    exercise_data = __exercise_data__
+class Teacher(abstract.LessonbasedTeacher):
     OK = 0
     ERR_PICKY = 1
     ERR_NO_INTERVALLS = 2
     ERR_NOTERANGE = 3
-    def __init__(self, exname, app, config):
-        abstract.Teacher.__init__(self, exname, app, config)
+    def __init__(self, exname, app):
+        abstract.LessonbasedTeacher.__init__(self, exname, app)
+        self.lessonfileclass = lessonfile.IdByNameLessonfile
         self.m_statistics = statistics.HarmonicIntervalStatistics(self)
         self.m_tonika = None
         self.m_interval = None
+    def enter_test_mode(self):
+        self.m_statistics.enter_test_mode()
+        count, t = lessonfile.parse_test_def(self.m_P.header.test)
+        self.m_test_questions = self.get_list('intervals') * count
+        random.shuffle(self.m_test_questions)
+        self.m_test_idx = -1
     def new_question(self):
         """
         Return values:
@@ -62,6 +60,15 @@ class Teacher(abstract.Teacher):
             gobject.source_remove(self.m_timeout_handle)
             self.m_timeout_handle = None
 
+        if self.m_app.m_test_mode:
+            self.m_test_idx += 1
+            self.m_interval = self.m_test_questions[self.m_test_idx]
+            #FIXME use tone pitch range from preferences window.
+            self.m_tonika = mpd.MusicalPitch()
+            self.m_tonika.randomize("f", "f'")
+            self.q_status = const.QSTATUS_NEW
+            return self.OK
+
         if self.get_bool('config/picky_on_new_question') \
                  and self.q_status in [const.QSTATUS_NEW, const.QSTATUS_WRONG]:
             return Teacher.ERR_PICKY
@@ -71,8 +78,8 @@ class Teacher(abstract.Teacher):
             return Teacher.ERR_NO_INTERVALLS
         last_question = self.m_interval
         last_tonika = self.m_tonika
-        L, H = utils.adjust_low_high_to_irange(self.get_string('lowest_tone'),
-           self.get_string('highest_tone'), self.get_list('intervals'))
+        L, H = utils.adjust_low_high_to_irange(self.get_string('user/lowest_pitch'),
+           self.get_string('user/highest_pitch'), self.get_list('intervals'))
         while 1:
             # in this loop we will try to make a question that is not the
             # same that the last one.
@@ -95,15 +102,19 @@ class Teacher(abstract.Teacher):
         """
         assert self.q_status not in (const.QSTATUS_NO, const.QSTATUS_GIVE_UP)
         if self.m_interval == answer:
-            if self.q_status == const.QSTATUS_NEW:
+            if self.q_status == const.QSTATUS_NEW \
+                    and not self.m_custom_mode:
                 self.m_statistics.add_correct(self.m_interval)
             self.maybe_auto_new_question()
             self.q_status = const.QSTATUS_SOLVED
             return 1
         else:
             if self.q_status == const.QSTATUS_NEW:
-                self.m_statistics.add_wrong(self.m_interval, answer)
+                if not self.m_custom_mode:
+                    self.m_statistics.add_wrong(self.m_interval, answer)
                 self.q_status = const.QSTATUS_WRONG
+            if self.m_app.m_test_mode:
+                self.maybe_auto_new_question()
     def give_up(self):
         """This function is only called *after* the user already has
         answered wrong once, so the statistics are already updated.
@@ -130,11 +141,11 @@ class Teacher(abstract.Teacher):
     def play_melodic(self):
         if self.q_status == const.QSTATUS_NO:
             return
-        if self.get_bool('override_default_instrument'):
-            instr_low = self.get_int('lowest_instrument')
-            instr_low_vel = self.get_int('lowest_instrument_velocity')
-            instr_high = self.get_int('highest_instrument')
-            instr_high_vel = self.get_int('highest_instrument_velocity')
+        if self.get_bool('config/override_default_instrument'):
+            instr_low = self.get_int('config/lowest_instrument')
+            instr_low_vel = self.get_int('config/lowest_instrument_velocity')
+            instr_high = self.get_int('config/highest_instrument')
+            instr_high_vel = self.get_int('config/highest_instrument_velocity')
         else:
             instr_low = instr_high = self.get_int('config/preferred_instrument')
             instr_low_vel = instr_high_vel \
@@ -156,7 +167,6 @@ class Gui(abstract.IntervalGui):
         ################
         # practice_box #
         ################
-        self.g_merge_id = None
         self.g_repeat_melodic = gu.bButton(self.action_area,
             _("Repeat _melodic"), self.repeat_melodic)
 
@@ -167,35 +177,24 @@ class Gui(abstract.IntervalGui):
         ##############
         # config_box #        
         ##############
-        frame = gtk.Frame(_("Ask for these intervals"))
+        self.g_ask_for_frame = frame = gtk.Frame(_("Ask for these intervals"))
         self.config_box.pack_start(frame, False)
 
-        self.g_interval_selector = widgets.IntervalCheckBox(
-                                          self.m_exname, 'intervals')
+        self.g_interval_selector = IntervalCheckBox(self.m_exname, 'intervals')
 
         def _ff(var):
-            self.on_end_practise()
-            self.on_start_practise()
+            if self.m_t.m_custom_mode:
+                # If we are running in custom mode, then the user can
+                # select himself what intervals to practise. And then
+                # we have to reset the exercise.
+                self.on_end_practise()
+                self.on_start_practise()
         self.add_watch('intervals', _ff)
         self.g_interval_selector.set_border_width(gnome.ui.PAD)
         frame.add(self.g_interval_selector)
         #------we need some space
         self.config_box.pack_start(gtk.HBox(), False,
                                    padding=gnome.ui.PAD_SMALL)
-        # -------------------------------------------------
-        hbox = gu.bHBox(self.config_box, False)
-        hbox.set_spacing(gnome.ui.PAD)
-        hbox.pack_start(gtk.Label(_("Try to keep question with range from")),
-                        False)
-        lowspin = widgets.NotenameSpinButton(self.get_string('lowest_tone'))
-        hbox.pack_start(lowspin, False)
-        gu.bLabel(hbox, _("to"), False)
-        highspin = widgets.NotenameSpinButton(self.get_string('highest_tone'))
-        hbox.pack_start(highspin, False)
-        self.m_notenamerange = widgets.nNotenameRangeController(lowspin, highspin,
-                           mpd.LOWEST_NOTENAME, mpd.HIGHEST_NOTENAME,
-                           self.m_t.m_exname, 'lowest_tone', 'highest_tone')
-
         # ------------------------------------------
         self._add_auto_new_question_gui(self.config_box)
         # ----------------------------------------------
@@ -214,9 +213,6 @@ class Gui(abstract.IntervalGui):
                     'disable_unused_intervals', _("_Disable unused buttons"))
         hbox.pack_start(self.g_disable_unused_buttons)
         # ------------ frame -------------------
-        self.g_instrument_configurator = widgets.InstrumentConfigurator(
-                                                             self.m_exname, 2)
-        self.config_box.pack_start(self.g_instrument_configurator, False)
         self.config_box.set_spacing(0)
         self.config_box.show_all()
         ##############
@@ -224,26 +220,7 @@ class Gui(abstract.IntervalGui):
         ##############
         self.setup_statisticsviewer(statisticsviewer.StatisticsViewer,
                                    _("Harmonic interval"))
-        ########
-        # Test #
-        ########
-        self.g_action_group = gtk.ActionGroup('HarmonicIntervalActions')
-        self.g_action_group.add_actions([
-            ('ExerciseMenu', None, _('_Exercise')),
-            ('TestEx', None, _('_Run test'), None, None, self.run_test),
-            ('HarmTestStatistics', None, _('_Test statistics'), None, None,
-             self.show_test_statistics),
-            ])
-        self.g_win.g_ui_manager.insert_action_group(self.g_action_group, 0)
         self.select_inputwidget()
-    def run_test(self, *v):
-        tw = intervaltest.HarmonicIntervalTestDialog()
-        tw.set_modal(True)
-        tw.show_all()
-        gtk.main()
-        self.g_test_stat_dlg = intervaltest.TestStatisticsDialog(self, _("Harmonic interval test results"), tw.m_test.get_test_results_dict(), False)
-        self.g_test_stat_dlg.show_all()
-        tw.destroy()
     def give_up(self, _o=None):
         if self.m_t.q_status == const.QSTATUS_WRONG:
             self.g_flashbar.push(_("The answer is: %s")
@@ -307,6 +284,11 @@ class Gui(abstract.IntervalGui):
         button, or when you use the key bindings. So it can be called even
         if the 'New interval' button is insensitive.
         """
+        if self.m_t.m_app.m_test_mode and self.m_t.is_test_complete():
+            self.do_test_complete()
+            return
+        if self.m_t.m_app.m_test_mode:
+            self.g_new_interval.hide()
         g = self.m_t.new_question()
         if g == Teacher.OK: # new question, everything is OK
             self.g_repeat.set_sensitive(True)
@@ -331,21 +313,27 @@ class Gui(abstract.IntervalGui):
         self.m_t.play_melodic()
         self.g_input.grab_focus_first_sensitive_button()
     def on_start_practise(self):
-        self.m_t.m_statistics.reset_session()
-        self.g_action_group.set_sensitive(True)
+        self.m_t.m_custom_mode = bool(not self.m_t.m_P.header.intervals)
+        if self.m_t.m_P.header.intervals:
+            self.set_list('intervals', self.m_t.m_P.header.intervals)
+        if self.m_t.m_P.header.lesson_heading:
+            self.set_lesson_heading(self.m_t.m_P.header.lesson_heading)
+        else:
+            self.set_lesson_heading(_("Identify the interval"))
+        self.set_bool('disable_unused_intervals', self.m_t.m_P.header.disable_unused_intervals)
+        if self.m_t.m_custom_mode:
+            self.g_ask_for_frame.show()
+        else:
+            self.m_t.m_statistics.reset_session()
+            self.g_ask_for_frame.hide()
         self.g_new_interval.grab_focus()
-        gobject.timeout_add(const.SHORT_WAIT, lambda self=self:
-            self.g_flashbar.flash(_("Click 'New interval' to begin.")))
-        self.g_merge_id = self.g_win.g_ui_manager.add_ui_from_string("""
-<menubar name='Menubar'>
-  <menu action='ExerciseMenu'>
-    <menuitem action='TestEx'/>
-    <menuitem action='HarmTestStatistics'/>
-  </menu>
-</menubar>""")
+        if self.m_t.m_app.m_test_mode:
+            gobject.timeout_add(const.SHORT_WAIT, lambda self=self:
+                self.g_flashbar.flash(_("Click 'Start test' to begin.")))
+        else:
+            gobject.timeout_add(const.SHORT_WAIT, lambda self=self:
+                self.g_flashbar.flash(_("Click 'New interval' to begin.")))
     def on_end_practise(self):
-        if self.g_merge_id:
-            self.g_win.g_ui_manager.remove_ui(self.g_merge_id)
         self.m_t.end_practise()
         self.g_new_interval.set_sensitive(True)
         self.g_repeat.set_sensitive(False)
@@ -353,3 +341,20 @@ class Gui(abstract.IntervalGui):
         self.g_give_up.set_sensitive(False)
         self.g_flashbar.clear()
         self.g_input.clear()
+    def enter_test_mode(self):
+        self.m_saved_q_auto = self.get_bool('new_question_automatically')
+        self.m_saved_s_new = self.get_float('seconds_before_new_question')
+        self.set_bool('new_question_automatically', True)
+        self.set_float('seconds_before_new_question', 0.5)
+        self.m_t.enter_test_mode()
+        self.g_new_interval.set_label(_("_Start test"))
+        self.g_repeat_melodic.hide()
+        self.g_give_up.hide()
+    def exit_test_mode(self):
+        self.set_bool('new_question_automatically', self.m_saved_q_auto)
+        self.set_float('seconds_before_new_question', self.m_saved_s_new)
+        self.m_t.exit_test_mode()
+        self.g_new_interval.show()
+        self.g_new_interval.set_label(_("_New interval"))
+        self.g_repeat_melodic.show()
+        self.g_give_up.show()
