@@ -15,24 +15,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-__exercise_data__ = {
-    'exercise_name': 'id-by-name',
-    'stock-icon': 'solfege-id-by-name',
-    'menupath': '/Menubar/FileMenu/MiscMenu',
-    'name': _("Id_entify by name"),
-}
-import exercise_setup
-exercise_setup.register_exercise(__exercise_data__)
-
-if __name__ == 'idbyname':
-    # pydoc ??
-    import sys
-    sys.path.append("src")
-    import i18n
-    i18n.setup_srcdir()
-
 import gtk, gnome, gobject
-import gu, widgets
+import gu
+from specialwidgets import QuestionNameButtonTable, QuestionNameCheckButtonTable
 import abstract, const, mpd, mpd.musicdisplayer
 import utils
 import statistics, statisticsviewer
@@ -42,19 +27,18 @@ import soundcard
 
 
 class Teacher(abstract.LessonbasedTeacher):
-    exercise_data = __exercise_data__
     OK = 0
     ERR_PICKY = 1
     ERR_NO_QUESTION = 2
-    def __init__(self, exname,  app, lesson_collection, lesson_file, config):
-        abstract.LessonbasedTeacher.__init__(self, exname, app,
-                lesson_collection, lesson_file, config)
+    def __init__(self, exname,  app):
+        abstract.LessonbasedTeacher.__init__(self, exname, app)
         self.lessonfileclass = lessonfile.IdByNameLessonfile
-        try: # try: because the file can have changed since startup
-            self.parse_lessonfile()
-        except dataparser.DataparserException, e:
-            self.m_app.m_ui.display_exception_message(e, __file__)
         self.m_statistics = statistics.LessonStatistics(self)
+    def enter_test_mode(self):
+        self.m_statistics.enter_test_mode()
+        self.m_P.enter_test_mode()
+    def exit_test_mode(self):
+        self.m_statistics.exit_test_mode()
     def give_up(self):
         self.q_status = const.QSTATUS_GIVE_UP
     def new_question(self):
@@ -65,6 +49,11 @@ class Teacher(abstract.LessonbasedTeacher):
             gobject.source_remove(self.m_timeout_handle)
             self.m_timeout_handle = None
 
+        if self.m_app.m_test_mode:
+            self.m_P.next_test_question()
+            self.q_status = const.QSTATUS_NEW
+            return self.OK
+
         if self.get_bool('config/picky_on_new_question') \
                  and self.q_status in [const.QSTATUS_NEW, const.QSTATUS_WRONG]:
             return Teacher.ERR_PICKY
@@ -80,16 +69,19 @@ class Teacher(abstract.LessonbasedTeacher):
         Return: 1 if correct, None if wrong
         """
         if answer == self.m_P.get_cname():
-            if self.q_status == const.QSTATUS_NEW:
+            if self.q_status == const.QSTATUS_NEW \
+                    and not self.get_bool('gui/expert_mode'):
                 self.m_statistics.add_correct(answer)
             self.maybe_auto_new_question()
             self.q_status = const.QSTATUS_SOLVED
             return 1
         else:
             if self.q_status == const.QSTATUS_NEW:
-                self.m_statistics.add_wrong(self.m_P.get_cname(),
-                                            answer)
+                if not self.get_bool('gui/expert_mode'):
+                    self.m_statistics.add_wrong(self.m_P.get_cname(), answer)
                 self.q_status = const.QSTATUS_WRONG
+            if self.m_app.m_test_mode:
+                self.maybe_auto_new_question()
 
 class Gui(abstract.LessonbasedGui):
     def __init__(self, teacher, window):
@@ -99,7 +91,7 @@ class Gui(abstract.LessonbasedGui):
         ################
         self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer(utils.play_tone)
         self.practise_box.pack_start(self.g_music_displayer, False)
-        self.g_bb = widgets.QuestionNameButtonTable(self.m_t.m_exname)
+        self.g_bb = QuestionNameButtonTable(self.m_t.m_exname)
         self.practise_box.pack_start(self.g_bb, False)
 
         self.g_flashbar = gu.FlashBar()
@@ -108,9 +100,9 @@ class Gui(abstract.LessonbasedGui):
         self.practise_box.set_spacing(gnome.ui.PAD)
 
         self.g_new = gu.bButton(self.action_area, _("_New"), self.new_question)
-        self.g_play_music = gu.bButton(self.action_area, _("Play music"),
+        self.g_play_music = gu.bButton(self.action_area, _("P_lay music"),
             lambda w: self.m_t.m_P.play_question())
-        self.g_display_music = gu.bButton(self.action_area, _("Display music"),
+        self.g_display_music = gu.bButton(self.action_area, _("_Display music"),
             self.show_music)
         self.g_repeat = gu.bButton(self.action_area, _("_Repeat"),
                                    self.repeat_question)
@@ -130,13 +122,12 @@ class Gui(abstract.LessonbasedGui):
         ##############
         self.config_box.set_border_width(12)
         self.config_box.set_spacing(18)
-        self.add_select_lessonfile_gui()
         self.add_random_transpose_gui()
         # -----------------------------------------
         self.g_select_questions_category_box, category_box= gu.hig_category_vbox(
             _("Questions to ask"))
         self.config_box.pack_start(self.g_select_questions_category_box, True)
-        self.g_select_questions = widgets.QuestionNameCheckButtonTable(self.m_t)
+        self.g_select_questions = QuestionNameCheckButtonTable(self.m_t)
         self.g_select_questions.initialize(4, 0)
         category_box.pack_start(self.g_select_questions, False)
         self.g_select_questions.show()
@@ -148,11 +139,8 @@ class Gui(abstract.LessonbasedGui):
         ##############
         self.setup_statisticsviewer(statisticsviewer.StatisticsViewer,
                                    _("Identify by name"))
-
-        self.update_gui_after_lessonfile_change()
         def _f(varname):
-            self.on_end_practise()
-            self.on_start_practise()
+            self.setup_action_area_buttons()
         self.add_watch('ask_for_names', _f)
     def show_music(self, widget=None):
         """
@@ -184,20 +172,28 @@ class Gui(abstract.LessonbasedGui):
                       self.m_t.m_P.header.labelformat, self.on_click)
         else:
             self.g_bb.initialize(0, 0)
-        if self.m_t.m_P and self.m_t.m_P.header.questions_selectable:
+        if self.m_t.m_P and self.get_bool('gui/expert_mode'):
             self.g_bb.ask_for_names_changed()
     def update_select_question_buttons(self):
-        if self.m_t.m_P and self.m_t.m_P.header.questions_selectable:
+        #FIXME duplicate code in src/chord.py
+        if self.get_bool('gui/expert_mode'):
             self.g_select_questions_category_box.show()
             self.g_select_questions.initialize(self.m_t.m_P.header.fillnum,
                                  self.m_t.m_P.header.filldir)
+            self.m_t.check_askfor()
             for question in self.m_t.m_P.iterate_questions_with_unique_names():
                 self.g_select_questions.add(question,
                       self.m_t.m_statistics.get_label_style())
         else:
             self.g_select_questions_category_box.hide()
             self.g_select_questions.initialize(0, 0)
-    def update_gui_after_lessonfile_change(self):
+    def setup_action_area_buttons(self):
+        """
+        Make the buttons visible or invisible depending
+        on the lesson file, and make the right set of buttons
+        sensitive for the first question.
+        """
+        self.g_new.set_sensitive(True)
         self.g_repeat.set_sensitive(False)
         self.g_repeat_slowly.set_sensitive(False)
         self.g_repeat_arpeggio.set_sensitive(False)
@@ -205,42 +201,38 @@ class Gui(abstract.LessonbasedGui):
         self.g_display_music.set_sensitive(False)
         self.g_play_music.set_sensitive(False)
         self.g_give_up.set_sensitive(False)
-        self.g_win.set_title("Solfege - " + self.get_pretty_name())
-        if self.m_t.m_P:
-            self.g_new.set_sensitive(True)
-            self.g_random_transpose.set_text(str(self.m_t.m_P.header.random_transpose))
-            if filter(lambda q: q['music'].m_musictype == 'chord',
-                      self.m_t.m_P.m_questions):
-                self.g_repeat_arpeggio.show()
-            else:
-                self.g_repeat_arpeggio.hide()
-            if self.m_t.m_P.header.have_repeat_slowly_button:
-                self.g_repeat_slowly.show()
-            else:
-                self.g_repeat_slowly.hide()
-            if self.m_t.m_P.header.at_question_start == 'show':
-                self.g_play_music.show()
-                self.g_repeat.hide()
-            else:
-                self.g_play_music.hide()
-                self.g_repeat.show()
-            if self.m_t.m_P.header.at_question_start == 'play':
-                self.g_display_music.show()
-            else:
-                self.g_display_music.hide()
-            if self.m_t.m_P.header.have_music_displayer:
-                self.g_music_displayer.show()
-                self.g_music_displayer.clear()
-                self.g_show.hide()
-            else:
-                self.g_music_displayer.hide()
-                self.g_show.show()
-            if self.m_t.m_P.header.questions_selectable:
-                self.m_t.check_askfor()
+        if filter(lambda q: q.is_transposable(), self.m_t.m_P.m_questions):
+            self.g_random_transpose_box.show()
         else:
-            self.g_new.set_sensitive(False)
-        self.update_select_question_buttons()
-        self.update_answer_buttons()
+            self.g_random_transpose_box.hide()
+        # If one or more of the questions is of musictype 'chord', then
+        # we need the "Repeat arpeggio" button.
+        if filter(lambda q: q['music'].m_musictype == 'chord',
+                  self.m_t.m_P.m_questions):
+            self.g_repeat_arpeggio.show()
+        else:
+            self.g_repeat_arpeggio.hide()
+        if self.m_t.m_P.header.have_repeat_slowly_button:
+            self.g_repeat_slowly.show()
+        else:
+            self.g_repeat_slowly.hide()
+        if self.m_t.m_P.header.at_question_start == 'show':
+            self.g_play_music.show()
+            self.g_repeat.hide()
+        else:
+            self.g_play_music.hide()
+            self.g_repeat.show()
+        if self.m_t.m_P.header.at_question_start == 'play':
+            self.g_display_music.show()
+        else:
+            self.g_display_music.hide()
+        if self.m_t.m_P.header.have_music_displayer:
+            self.g_music_displayer.show()
+            self.g_music_displayer.clear()
+            self.g_show.hide()
+        else:
+            self.g_music_displayer.hide()
+            self.g_show.show()
     def give_up(self, _o=None):
         if self.m_t.q_status == const.QSTATUS_WRONG:
             self.g_flashbar.push(_("The answer is: %s") % self.m_t.m_P.get_name())
@@ -260,6 +252,8 @@ class Gui(abstract.LessonbasedGui):
             else:
                 self.g_flashbar.flash(_("Right click is not allowed for this lesson file."))
     def on_right_click(self, button):
+        if self.m_t.m_app.m_test_mode:
+            return
         if self.m_t.q_status == const.QSTATUS_NO:
             self.g_flashbar.flash(_("Click 'New' to begin."))
             return
@@ -307,6 +301,11 @@ class Gui(abstract.LessonbasedGui):
         """
         The new button should be insensitive if we have no lesson file.
         """
+        if self.m_t.m_app.m_test_mode and self.m_t.m_P.is_test_complete():
+            self.do_test_complete()
+            return
+        if self.m_t.m_app.m_test_mode:
+            self.g_new.hide()
         def exception_cleanup():
             self.m_t.q_status = const.QSTATUS_NO
             soundcard.synth.stop()
@@ -365,6 +364,11 @@ class Gui(abstract.LessonbasedGui):
         self.g_bb.grab_focus_first_button()
         self.m_t.m_P.play_question_slowly()
     def on_start_practise(self):
+        self.setup_action_area_buttons()
+        self.update_answer_buttons()
+        self.update_select_question_buttons()
+        self.set_lesson_heading(self.m_t.m_P.header.lesson_heading)
+        self.g_random_transpose.set_text(str(self.m_t.m_P.header.random_transpose))
         self.g_flashbar.require_size([
             _("Right click is not allowed for this lesson file."),
             _("You should try to guess before right-clicking."),
@@ -374,10 +378,15 @@ class Gui(abstract.LessonbasedGui):
             _("You have to select some questions to practise."),
         ])
         self.g_new.grab_focus()
-        self.m_t.m_statistics.reset_session()
+        if not self.get_bool('gui/expert_mode'):
+            self.m_t.m_statistics.reset_session()
         self.g_music_displayer.clear()
-        gobject.timeout_add(const.SHORT_WAIT, lambda self=self:
-            self.g_flashbar.flash(_("Click 'New' to begin.")))
+        if self.m_t.m_app.m_test_mode:
+            gobject.timeout_add(const.SHORT_WAIT, lambda self=self:
+                self.g_flashbar.flash(_("Click 'Start test' to begin.")))
+        else:
+            gobject.timeout_add(const.SHORT_WAIT, lambda self=self:
+                self.g_flashbar.flash(_("Click 'New' to begin.")))
     def on_end_practise(self):
         self.m_t.end_practise()
         self.g_repeat.set_sensitive(False)
@@ -389,3 +398,22 @@ class Gui(abstract.LessonbasedGui):
         self.g_new.set_sensitive(True)
         self.g_give_up.set_sensitive(False)
         self.g_flashbar.clear()
+    def enter_test_mode(self):
+        self.m_saved_q_auto = self.get_bool('new_question_automatically')
+        self.m_saved_s_new = self.get_float('seconds_before_new_question')
+        self.set_bool('new_question_automatically', True)
+        self.set_float('seconds_before_new_question', 0.5)
+        self.m_t.enter_test_mode()
+        self.g_give_up.hide()
+        self.g_new.set_label(_("_Start test"))
+        self.g_show.hide()
+        self.g_repeat_arpeggio.hide()
+        self.g_repeat_slowly.hide()
+    def exit_test_mode(self):
+        self.set_bool('new_question_automatically', self.m_saved_q_auto)
+        self.set_float('seconds_before_new_question', self.m_saved_s_new)
+        self.m_t.exit_test_mode()
+        self.g_new.set_label(_("_New"))
+        self.g_new.show()
+        self.on_start_practise()
+        self.g_repeat_slowly.show()
