@@ -166,7 +166,7 @@ class InstrumentSelector(ReadOnlyInstrumentSelector):
         self.set_int(self.m_name+'_velocity', p.get_value_as_int())
 
 class InstrumentConfigurator(gtk.VBox, cfg.ConfigUtils):
-    def __init__(self, exname, num):
+    def __init__(self, exname, num, labeltext):
         gtk.VBox.__init__(self)
         #cfg.ConfigUtils.__init__(self, exname)
         cfg.ConfigUtils.__dict__['__init__'](self, exname)
@@ -174,7 +174,7 @@ class InstrumentConfigurator(gtk.VBox, cfg.ConfigUtils):
         self.m_num = num
         self.g_override_default_instrument_checkbutton \
             = gu.nCheckButton(exname, 'override_default_instrument',
-                 _("_Change midi instrument for this exercise"),
+                labeltext,
                  callback=self.update_instrument_override)
         self.pack_start(self.g_override_default_instrument_checkbutton,
                         False)
