@@ -19,10 +19,6 @@
 
 copyright = "Copyright © 1999-2005 Tom Cato Amundsen <tca@gnu.org>"
 
-import __builtin__
-__builtin__.app_running = True
-__builtin__.__exdata__ = []
-
 import sys
 import os
 import time
@@ -237,8 +233,14 @@ class ResolveLessonIdCrashDlg(gtk.Dialog):
                 g = self.radio_dict[self.radio_dict.keys()[0]]
             else:
                 g = None
-            radiobutton = gtk.RadioButton(g, "%s %s" % (info['filename'], info['timestr']))
+            ss = "%s %s" % (info['filename'], info['timestr'])
+            if not isinstance(ss, unicode):
+                ss = ss.decode(locale.nl_langinfo(locale.CODESET), 'replace')
+            radiobutton = gtk.RadioButton(g, ss)
+            radiobutton.show()
             radiobutton.set_data('fn', info['filename'])
+            radiobutton.set_data('lessonfile', info['lessonfile'])
+            radiobutton.set_data('lessoncollection', info['lessoncollection'])
             self.radio_dict[info['mtime']] = radiobutton
             vbox.pack_start(radiobutton)
         self.radio_dict[min(self.radio_dict.keys())].set_active(True)
@@ -260,6 +262,7 @@ class MusicViewerWindow(gtk.Dialog):
         self.g_music_displayer.display(music, fontsize)
 
 class MainWin(gnome.ui.App, cfg.ConfigUtils):
+    default_learning_tree = ('solfege', 'learningtree.txt')
     def __init__(self, options, datadir, lessonfile_manager):
         gnome.ui.App.__init__(self, 'solfege', 'GNU Solfege')
         self.icons = stock.SolfegeIconFactory(self, datadir)
@@ -272,6 +275,7 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
         self.connect('destroy', self.quit_program)
         self.connect('key_press_event', self.on_key_press_event)
         self.g_about_window = None
+        self.g_learning_tree_editor = None
         self.main_box = gtk.VBox()
         self.main_box.show()
         self.set_contents(self.main_box)
@@ -286,8 +290,6 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
                                'history_reload_ak': self.history_reload}
         self.m_app = app.SolfegeApp(options, self, lessonfile_manager)
         self.g_ui_manager = gtk.UIManager()
-        self.m_tree = learning_tree_editor.LearningTree(self.m_app.lessonfile_manager)
-        self.m_tree.load('learningtree.txt')
         self.m_action_groups = {
             'Practise': gtk.ActionGroup('Practise'),
             'Exit': gtk.ActionGroup('Exit'),
@@ -297,9 +299,32 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
             self.g_ui_manager.insert_action_group(a, 1)
         self.setup_menu()
         self.m_app.handle_href('welcome.html')
+    def change_learning_tree(self, tree_fn):
+        """
+        Change to a different learning tree.
+        """
+        self.set_list('app/learningtree', tree_fn)
+        self.on_learning_tree_changed()
     def on_learning_tree_changed(self, *v):
+        """
+        We call this when we have changed the current tree or
+        switched tree.
+        """
         self.m_tree = learning_tree_editor.LearningTree(self.m_app.lessonfile_manager)
-        self.m_tree.load('learningtree.txt')
+        try:
+            loc, filename = self.get_list("app/learningtree")
+        except ValueError:
+            loc, filename = self.default_learning_tree
+        try:
+            if loc == 'solfege':
+                self.m_tree.load(filename)
+            else:
+                assert loc == 'user'
+                self.m_tree.load(os.path.join(os.path.expanduser('~'), '.solfege', 'learningtrees', filename))
+        except IOError:
+            self.m_tree.load(self.default_learning_tree[1])
+            self.set_list('app/learningtree', self.default_learning_tree)
+            self.dialog_ok(_("Learning tree '%s' not found. Using default tree." % filename))
         self.create_practise_and_test_menu()
     def setup_menu(self):
         self.m_action_groups['Exit'].add_actions([
@@ -367,20 +392,35 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
         self.g_ui_manager.get_accel_group().connect_group(gtk.keysyms.KP_Add,
            gtk.gdk.CONTROL_MASK, 0,
            lambda w, a, b, c: self.box_dict['docviewer'].m_htmlwidget.g_view.zoom_in())
-        for e in __exdata__:
-            if 'menupath' in e:
-                a = gtk.Action(e['action'], e['name'], e['exercise_pretty_name'], e['stock-icon'])
-                self.m_action_groups['NotExit'].add_action(a)
-                a.connect('activate', lambda o, e=e:
-                    self.m_app.handle_href('solfege:/%s/%s' % (
-                      e['exercise_name'], e['exercise_config'])))
-                if not self.g_ui_manager.get_widget(e['menupath']):
-                    print "Miss:", e['menupath'], e['exercise_name']
-                self.g_ui_manager.add_ui(self.g_ui_manager.new_merge_id(),
-                   e['menupath'], e['name'], e['action'],
-                   gtk.UI_MANAGER_MENUITEM, False)
         self.set_menus(self.g_ui_manager.get_widget('/Menubar'))
         self.m_help_on_current_merge_id = None
+    def create_learning_trees_menu(self):
+        if self.m_learning_trees_merge_id:
+            self.g_ui_manager.remove_ui(self.m_learning_trees_merge_id)
+        try:
+            v = os.listdir(os.path.expanduser("~/.solfege/learningtrees"))
+        except OSError:
+            v = []
+        if not v:
+            return
+        actions = [('LearningTreesMenu', None, _('Learning tree')),
+            ('DefaultTree', None, _("Default learning tree"),
+            None, None,
+           lambda o: self.change_learning_tree(self.default_learning_tree)
+            )]
+        s = """
+<menubar name='Menubar'>
+ <menu action='FileMenu'>
+  <menu action='LearningTreesMenu' position='top'>
+    <menuitem action='DefaultTree'/>"""
+        for tree_fn in v:
+            s += "<menuitem action='LTree-%s'/>" % tree_fn
+            actions.append(('LTree-%s' % tree_fn, None,
+                    tree_fn, None, None,
+                    lambda o, t=tree_fn: self.change_learning_tree(('user', t))))
+        s += "</menu></menu></menubar>"
+        self.m_action_groups['NotExit'].add_actions(actions)
+        self.m_learning_trees_merge_id = self.g_ui_manager.add_ui_from_string(s)
     def create_practise_and_test_menu(self):
         """
         Create the Practise menu.
@@ -397,8 +437,8 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
         for topic in self.m_tree.m_topics:
             s += "<menu action='%s'>" % topic['name']
             tests += "<menu action='TEST_%s'>" % topic['name']
-            actions.append((topic['name'], None, topic['name']))
-            actions.append(('TEST_%s' % topic['name'], None, topic['name']))
+            actions.append((topic['name'], None, _(topic['name'])))
+            actions.append(('TEST_%s' % topic['name'], None, _(topic['name'])))
             for lesson_id in topic['lessons']:
                 if self.m_tree.m_visibilities[lesson_id] > self.m_tree.m_visibility:
                     continue
@@ -487,15 +527,18 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
             msg = _("Exception catched in %s:%i\n") % (err_file, sys.exc_info()[2].tb_lineno) + "\n"
         else:
             msg = ""
-        msg += str(exception)
+        exception_str = unicode(exception)
+        if not isinstance(exception_str, unicode):
+            exception_str = exception_str.decode(locale.nl_langinfo(locale.CODESET), 'replace')
+        msg = "%s %s" %(msg, exception)
         m = gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
                               gtk.BUTTONS_CLOSE, msg)
         m.set_title(sys.exc_info()[0].__name__)
         m.run()
         m.destroy()
-    def display_error_message(self, msg, err_pos=None):
-        if err_pos:
-            msg = _("Error found in %s:\n") % err_pos + "\n" + msg
+    def display_error_message(self, msg):
+        if not isinstance(msg, unicode):
+            msg = msg.decode(locale.nl_langinfo(locale.CODESET), 'replace')
         m = gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
                               gtk.BUTTONS_CLOSE, None)
         m.set_markup(msg)
@@ -521,7 +564,7 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
                   'Giovanni Chierico %s' % _("(some lessonfiles)"),
                   'Michael Becker %s' % _("(some lessonfiles)"),
                   'Joe Lee %s' % _("(sound code for the MS Windows port)"),
-                  'Steve Lee %s' % _("ported winmidi.c to gcc)"),
+                  'Steve Lee %s' % _("(ported winmidi.c to gcc)"),
                   'Thibaus Cousin %s' % _("(spec file for SuSE 8.2)"),
                   'David Coe %s' %_("(spec file cleanup)"),
                   'David Petrou %s' % _("(testing and portability fixes for FreeBSD)"),
@@ -539,17 +582,36 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
         self.g_about_window.destroy()
         self.g_about_window = None
     def do_tree_editor(self, *v):
-        win = learning_tree_editor.Window(self.m_app)
-        win.load_file('learningtree.txt')
+        # Raise it if it already exist.
+        if self.g_learning_tree_editor:
+            self.g_learning_tree_editor.present()
+            return
+        # And if it don't exist, then we have to create it.
+        self.g_learning_tree_editor = win = learning_tree_editor.Window(self.m_app)
+        try:
+            loc, filename = self.get_list("app/learningtree")
+        except ValueError:
+            loc, filename = self.default_learning_tree
+        if loc == 'solfege':
+            win.load_file(filename)
+        else:
+            assert loc == 'user'
+            win.load_file(os.path.join(os.path.expanduser('~'), '.solfege', 'learningtrees', filename))
         win.show()
         x, y = win.g_topic_scrollwin.size_request()
         win.g_lesson_scrollwin.set_size_request(x, int(y*1.6))
         win.g_deps_scrollwin.set_size_request(x, int(y*1.6))
     def post_constructor(self):
+        global splash_win
         self.m_practise_and_test_merge_id = None
-        self.create_practise_and_test_menu()
+        self.m_learning_trees_merge_id = None
+        self.on_learning_tree_changed()
+        self.create_learning_trees_menu()
         self.box_dict['docviewer'].grab_focus()
         if self.m_app.m_sound_init_exception is not None:
+            if splash_win:
+                splash_win.destroy()
+                splash_win = None
             self.m_app.display_sound_init_error_message(self.m_app.m_sound_init_exception)
     def activate_exercise(self, module, urlobj=None):
         if self.m_viewer:
@@ -641,6 +703,8 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
     def on_zoom_in(self, *v):
         self.box_dict['docviewer'].m_htmlwidget.g_view.zoom_in()
     def quit_program(self, w=None):
+        if self.g_learning_tree_editor:
+            self.g_learning_tree_editor.close_window()
         self.m_app.quit_program()
         gtk.main_quit()
     def display_in_musicviewer(self, music):
@@ -671,25 +735,29 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
         self.create_practise_and_test_menu()
 
 def handle_lesson_id_crash(lessonfile_manager):
-    has_crash = False
     for lesson_id in lessonfile_manager.iterate_duplicated_lesson_id():
-        has_crash = True
         if splash_win:
             splash_win.hide()
         dlg = ResolveLessonIdCrashDlg(lessonfile_manager, lesson_id)
         ret = dlg.run()
         if ret == gtk.RESPONSE_OK:
             for b in dlg.radio_dict.values():
+                # Create a new lesson_id and parse them into m_uiddb
                 if not b.get_active():
                     f = lessonfile.LessonIdParser(b.get_data('fn'))
                     f.new_lesson_id()
+                    lessonfile_manager.parse_into_uiddb(
+                        b.get_data('lessoncollection'),
+                        b.get_data('lessonfile'))
+                # and remove the duplicate entries from the entry we
+                # want to keep.
+                else:
+                    lessonfile_manager.delete_not_fn(lesson_id, b.get_data('fn'))
         elif ret == gtk.RESPONSE_CANCEL:
-            lessonfile_manager.ignore_files_with_lesson_id(lesson_id)
+            lessonfile_manager.ignore_duplicates_with_lesson_id(lesson_id)
         dlg.destroy()
     if splash_win:
         splash_win.show()
-    if has_crash:
-        lessonfile_manager.parse()
 
 if splash_win:
     splash_win.show_progress("running gnome.program_init")
@@ -708,6 +776,7 @@ if splash_win:
 lessonfile_manager = lessonfile.LessonFileManager()
 
 handle_lesson_id_crash(lessonfile_manager)
+lessonfile_manager.create_lessonfile_index()
 
 if splash_win:
     splash_win.show_progress("creating MainWin")
