@@ -59,7 +59,7 @@ class ReportBugWindow(gtk.Dialog):
             False)
         self.g_mangled_email = gtk.Label()
         self.vbox.pack_start(
-            gu.hig_label_widget(_("_Mangled email addr:"), self.g_mangled_email,
+            gu.hig_label_widget(_("Mangled email address:"), self.g_mangled_email,
                 sizegroup), False)
         self.g_email.set_text(cfg.get_string('user/email'))
         self.g_email.connect('changed', self.on_update_mangle)
